@@ -21,12 +21,15 @@ from django import template
 from django.conf import settings
 from django.core.exceptions import PermissionDenied
 from django.db import models
+from django.db.models import Q
+
 from django.db.models.related import RelatedObject
 from django.db.models.fields.related import ForeignKey
 from django.shortcuts import render_to_response
 from django.contrib import admin
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
+from django.contrib.admin.filterspecs import FilterSpec
 from django.contrib.admin.views.main import ChangeList, ERROR_FLAG
 from django.contrib.admin.options import IncorrectLookupParameters
 from django.contrib.admin.util import unquote, flatten_fieldsets
@@ -44,18 +47,24 @@ from django.utils.html import escape
 from django.utils.importlib import import_module
 from django.utils.text import capfirst
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext
+from django.utils.translation import ugettext, get_language
 from django.utils.translation import ugettext_lazy as _
 
 from autoreports.admin import ReportAdmin
 from cmsutils.forms.widgets import AJAXAutocompletionWidget, ReadOnlyWidget
 from oembed.models import ProviderRule, StoredOEmbed
+from announcements.models import Announcement
+from announcements.admin import AnnouncementAdmin as AnnouncementDefaultAdmin
+from merengue.base.forms import AnnouncementAdminForm
+from notification.models import NoticeType, NoticeSetting, Notice
+from notification.admin import NoticeTypeAdmin, NoticeSettingAdmin, NoticeAdmin
 from transmeta import (canonical_fieldname, get_all_translatable_fields,
                        get_real_fieldname_in_each_language,
-                       get_fallback_fieldname)
+                       get_fallback_fieldname, get_real_fieldname)
 
 from merengue.base.adminsite import site
 from merengue.base.admin_utils import get_deleted_contents
+from merengue.base.filterspecs import ClassnameFilterSpec
 from merengue.base.forms import AdminBaseContentOwnersForm, BaseAdminModelForm
 from merengue.base.models import BaseContent, ContactInfo
 from merengue.base.widgets import CustomTinyMCE, RelatedBaseContentWidget
@@ -68,6 +77,12 @@ from genericforeignkey.admin import GenericAdmin
 LOADING = False
 
 
+# Don't call register but insert it at the beginning of the registry
+# otherwise, the AllFilterSpec will be taken first
+FilterSpec.filter_specs.insert(0, (lambda f: f.name == 'class_name',
+                                   ClassnameFilterSpec))
+
+
 def register_app(app_name, admin_site=None):
     admin_function(function_name='register', app_name=app_name,
                    admin_site=admin_site)
@@ -278,6 +293,8 @@ class RelatedURLsModelAdmin(admin.ModelAdmin):
         def wrap(view):
 
             def wrapper(*args, **kwargs):
+                #if isinstance(self, RelatedModelAdmin):
+                    #kwargs['parent_model_admin'] = self
                 return self.admin_site.admin_view(view)(*args, **kwargs)
             return update_wrapper(wrapper, view)
 
@@ -300,11 +317,14 @@ class RelatedURLsModelAdmin(admin.ModelAdmin):
         )
         return urlpatterns
 
-    def parse_path(self, request, pathstr, extra_context=None, basecontent=None):
+    def parse_path(self, request, pathstr, extra_context=None, basecontent=None, parent_model_admin=None, parent_object=None):
         extra_context = extra_context or {}
         path = pathstr.split('/')
         if len(path) == 1:
-            return self.change_view(request, path[0], extra_context)
+            if isinstance(self, RelatedModelAdmin):
+                return self.change_view(request, path[0], extra_context, parent_model_admin, parent_object)
+            else:
+                return self.change_view(request, path[0], extra_context)
         object_id = path[0]
         basecontent = self._get_base_content(request, object_id)
         tool_name = path[1]
@@ -322,6 +342,13 @@ class RelatedURLsModelAdmin(admin.ModelAdmin):
                     resolved = pattern.resolve(pathstr)
                     if resolved:
                         callback, args, kwargs = resolved
+                        # add ourselves as parent model admin to be referred from child model admin
+                        # add also parent object to be referred also in child model if needed
+                        if callback.func_name in ('changelist_view', 'change_view',
+                                                  'add_view', 'history_view',
+                                                  'parse_path'):
+                            kwargs['parent_model_admin'] = self
+                            kwargs['parent_object'] = basecontent
                         return callback(request, *args, **kwargs)
         raise Http404
 
@@ -398,6 +425,11 @@ class BaseAdmin(GenericAdmin, ReportAdmin, RelatedURLsModelAdmin):
     def formfield_for_dbfield(self, db_field, **kwargs):
         field = super(BaseAdmin, self).formfield_for_dbfield(db_field, **kwargs)
         db_fieldname = canonical_fieldname(db_field)
+        if field:
+            field.trans_candidate = getattr(db_field, 'original_fieldname', False) and True
+            field.canonical = db_fieldname
+            field.fallback = get_fallback_fieldname(db_fieldname)
+            field.current = get_real_fieldname(db_fieldname, get_language()) == db_field.name
 
         # tinymce editor for html fields
         if db_fieldname in self.html_fields:
@@ -424,7 +456,8 @@ class BaseAdmin(GenericAdmin, ReportAdmin, RelatedURLsModelAdmin):
 
         if field and isinstance(db_field, ForeignKey):
             if db_field.related.parent_model == BaseContent:
-                field.widget = RelatedBaseContentWidget(field.widget, field.widget.rel, field.widget.admin_site)
+                request = kwargs.get('request', None)
+                field.widget = RelatedBaseContentWidget(field.widget, field.widget.rel, field.widget.admin_site, request=request)
         return field
 
     def get_actions(self, request):
@@ -486,6 +519,9 @@ class BaseAdmin(GenericAdmin, ReportAdmin, RelatedURLsModelAdmin):
         """
         Hack for saving related object from edit_related admin option
         """
+        if hasattr(obj, 'last_editor'):
+            obj.last_editor = request.user
+
         super(BaseAdmin, self).save_model(request, obj, form, change)
         if change and self.edit_related:
             for related_field in self.edit_related:
@@ -570,6 +606,22 @@ class BaseAdmin(GenericAdmin, ReportAdmin, RelatedURLsModelAdmin):
         extra_context = self._base_update_extra_context(extra_context)
         return super(BaseAdmin, self).delete_view(request, object_id, extra_context)
 
+    def object_tools(self, request, mode, url_prefix):
+        """ Object tools for the model admin. mode can be "change", "add" or "list" """
+        tools = []
+        if mode == 'change':
+            tools.append(
+                {'url': url_prefix + 'history/', 'label': ugettext('History'), 'class': 'historylink', 'permission': 'change'},
+            )
+        elif mode == 'list':
+            tools.extend([
+                {'url': url_prefix + 'add/', 'label': ugettext('Add %s') % force_unicode(self.model._meta.verbose_name),
+                 'class': 'addlink', 'permission': 'add'},
+                {'url': url_prefix + 'report/quick/', 'label': ugettext('Quick Report'), 'class': 'quickreportlink'},
+                {'url': url_prefix + 'report/advance/', 'label': ugettext('Advanced Report'), 'class': 'advancedreportlink'},
+            ])
+        return tools
+
 
 class BaseCategoryAdmin(BaseAdmin):
     ordering = (get_fallback_fieldname('name'), )
@@ -648,9 +700,16 @@ class StatusControlProvider(object):
         options = set()
         all_options = set(settings.STATUS_LIST)
 
-        if hasattr(obj, 'owners'):
-            if not obj or user in obj.owners.all():
-                options = options.union([o for o in all_options if o[0] in ('draft', 'pending')])
+        original_obj = obj
+        # if there's not an object yet, we'll try to get the permissions for the section
+        if not obj:
+            try:
+                obj = self.basecontent
+            except AttributeError:
+                pass
+        if perms_api.has_permission(obj, user, 'edit'):
+            options = options.union([o for o in all_options if o[0] in ('draft', 'pending')])
+
         # Remember that superuser has all the perms
         if perms_api.has_permission(obj, user, 'can_draft'):
             options = options.union([o for o in all_options if o[0] == 'draft'])
@@ -658,6 +717,10 @@ class StatusControlProvider(object):
             options = options.union([o for o in all_options if o[0] == 'pending'])
         if perms_api.has_permission(obj, user, 'can_published'):
             options = options.union([o for o in all_options if o[0] == 'published'])
+        if original_obj and hasattr(original_obj, 'status'):
+            status_list = [o[0] for o in options]
+            if original_obj.status == 'published' and 'published' not in status_list:
+                return None
         return options
 
 
@@ -685,7 +748,7 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
         urls = super(BaseContentAdmin, self).get_urls()
         # override objectpermissions root path
         my_urls = patterns('',
-            (r'^([^/]+)/permissions/$', self.admin_site.admin_view(self.change_roles_permissions)))
+            (r'^([^/]+)/permissions/$', self.admin_site.admin_view(self.changelist_view)))
 
         return my_urls + urls
 
@@ -788,10 +851,16 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
             options = self._get_status_options(user, obj)
             if options:
                 form.base_fields['status'].choices = options
-                if 'pending' in [o[0] for o in options]:
-                    form.base_fields['status'].initial = 'pending'
+                default_status = getattr(settings, 'DEFAULT_STATUS', 'draft')
+                if default_status in [o[0] for o in options]:
+                    form.base_fields['status'].initial = default_status
             else:
-                form.base_fields.pop('status')
+                set_field_read_only(form.base_fields['status'], 'status', obj)
+        if 'owners' in keys:
+            owners_field = form.base_fields['owners']
+            if owners_field.initial is None:
+                # user automatically get owner of this object
+                owners_field.initial = (request.user.id, )
         if obj and obj.no_changeable_fields:
             no_changeable_fields = obj.no_changeable_fields
             for name in no_changeable_fields:
@@ -813,9 +882,6 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
 
         super(BaseContentAdmin, self).save_model(request, obj, form, change)
 
-        # user automatically get owner of this object
-        obj.owners.add(request.user)
-
     def formfield_for_dbfield(self, db_field, **kwargs):
         field = super(BaseContentAdmin, self).formfield_for_dbfield(db_field, **kwargs)
         db_fieldname = canonical_fieldname(db_field)
@@ -852,7 +918,7 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
 
         # perms_needed
 
-        if request.POST: # The user has already confirmed the deletion.
+        if request.POST:  # The user has already confirmed the deletion.
             if perms_needed or objects_without_delete_perm:
                 raise PermissionDenied
             obj_display = force_unicode(obj)
@@ -885,6 +951,12 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
         ], context, context_instance=context_instance)
 
     def select_changelist_view(self, request, extra_context=None):
+        widget_id = request.GET.get('widget_id', None)
+        if widget_id:
+            del(request.GET['widget_id'])
+            extra_query = request.session.get(widget_id, {})
+            if extra_query:
+                request.GET.update(extra_query)
         extra_context = self._base_update_extra_context(extra_context)
         opts = self.model._meta
         app_label = opts.app_label
@@ -904,6 +976,10 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
             return HttpResponseRedirect(request.path + '?' + ERROR_FLAG + '=1')
         cl.formset = None
         cl.params.update({'for_select': 1})
+        if widget_id:
+            cl.params.update({'widget_id': widget_id})
+            for key in extra_query.keys():
+                del(cl.params[key])
         context = {
             'title': cl.title,
             'is_popup': cl.is_popup,
@@ -938,10 +1014,22 @@ class BaseContentViewAdmin(BaseContentAdmin):
     """ An special admin to find and edit all site contents """
 
     list_display = ('admin_absolute_url', ) + BaseContentAdmin.list_display[1:]
+    list_filter = BaseContentAdmin.list_filter + ('class_name', )
 
     def has_add_permission(self, request):
         return False
 
+    def lookup_allowed(self, lookup):
+        is_allowed = super(BaseContentViewAdmin, self).lookup_allowed(lookup)
+        return is_allowed or lookup == u'id__in'
+
+    def queryset(self, request):
+        qs = super(BaseContentAdmin, self).queryset(request)
+        if perms_api.has_global_permission(request.user, 'manage_portal'):
+            return qs
+        elif self.has_change_permission(request):
+            return qs.filter(Q(owners=request.user) | Q(sections__owners=request.user))
+
 
 class RelatedModelAdmin(BaseAdmin):
     """
@@ -970,6 +1058,9 @@ class RelatedModelAdmin(BaseAdmin):
     reverse_related_field = None  # For m2m with through class
     one_to_one = False
     manage_contents = False
+    change_form_template = 'admin/related_change_form.html'
+    change_list_template = 'admin/related_change_list.html'
+    object_history_template = 'admin/related_object_history.html'
 
     def __init__(self, *args, **kwargs):
         super(RelatedModelAdmin, self).__init__(*args, **kwargs)
@@ -988,6 +1079,7 @@ class RelatedModelAdmin(BaseAdmin):
         def wrap(view):
 
             def wrapper(*args, **kwargs):
+                kwargs['model_admin'] = self
                 return self.admin_site.admin_view(view)(*args, **kwargs)
             return update_wrapper(wrapper, view)
 
@@ -1000,7 +1092,7 @@ class RelatedModelAdmin(BaseAdmin):
         urlpatterns += super(RelatedModelAdmin, self).get_urls()
         return urlpatterns
 
-    def _update_extra_context(self, request, extra_context=None):
+    def _update_extra_context(self, request, extra_context=None, parent_model_admin=None, parent_object=None):
         extra_context = extra_context or {}
         #basecontent = self._get_base_content(request)
         basecontent_type_id = ContentType.objects.get_for_model(self.basecontent).id
@@ -1009,7 +1101,11 @@ class RelatedModelAdmin(BaseAdmin):
                               'basecontent_opts': self.basecontent._meta,
                               'basecontent_type_id': basecontent_type_id,
                               'inside_basecontent': True,
-                              'selected': self.tool_name})
+                              'selected': self.tool_name,
+                              'model_admin': self,
+                              'parent_model_admin': parent_model_admin,
+                              'parent_object': parent_object,
+                              })
         return extra_context
 
     def is_created_one_to_one_object(self):
@@ -1018,7 +1114,7 @@ class RelatedModelAdmin(BaseAdmin):
             return obj[0]
         return None
 
-    def ajax_changelist_view(self, request):
+    def ajax_changelist_view(self, request, model_admin=None, parent_model_admin=None, parent_object=None):
         if not self.has_change_permission(request, None):
             raise PermissionDenied
         contents = [{'name': unicode(i), 'url': i.get_admin_absolute_url()} for i in self.queryset(request)]
@@ -1027,8 +1123,8 @@ class RelatedModelAdmin(BaseAdmin):
                                       'message': ugettext('No contents found')})
         return HttpResponse(json_dict, mimetype='text/plain')
 
-    def changelist_view(self, request, extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def changelist_view(self, request, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         if self.one_to_one:
             obj_created = self.is_created_one_to_one_object()
             if obj_created:
@@ -1043,24 +1139,24 @@ class RelatedModelAdmin(BaseAdmin):
             basecontent = self.basecontent
         return base_qs.filter(**{self.related_field: basecontent})
 
-    def add_view(self, request, form_url='', extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def add_view(self, request, form_url='', extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         if self.one_to_one:
             obj_created = self.is_created_one_to_one_object()
             if obj_created:
                 return HttpResponseRedirect('%s../%s' % (request.get_full_path(), obj_created.pk))
         return super(RelatedModelAdmin, self).add_view(request, form_url, extra_context)
 
-    def change_view(self, request, object_id, extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def change_view(self, request, object_id, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         return super(RelatedModelAdmin, self).change_view(request, object_id, extra_context)
 
-    def delete_view(self, request, object_id, extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def delete_view(self, request, object_id, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         return super(RelatedModelAdmin, self).delete_view(request, object_id, extra_context)
 
-    def history_view(self, request, object_id, extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def history_view(self, request, object_id, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         return super(RelatedModelAdmin, self).history_view(request, object_id, extra_context)
 
     def save_form(self, request, form, change):
@@ -1107,6 +1203,13 @@ class RelatedModelAdmin(BaseAdmin):
         if self.related_field in form.base_fields.keys():
             form.base_fields.pop(self.related_field)
 
+    def object_tools(self, request, mode, url_prefix):
+        """ Object tools for the model admin """
+        return BaseAdmin.object_tools(self, request, mode, url_prefix)
+
+    def has_add_permission(self, request):
+        return perms_api.has_permission(self.basecontent, request.user, 'edit')
+
 
 class BaseContentRelatedContactInfoAdmin(RelatedModelAdmin):
     tool_name = 'contact_info'
@@ -1190,7 +1293,7 @@ class OrderableRelatedModelAdmin(RelatedModelAdmin):
 
       >>> site.register_related(Page, PageOrderableRelatedAdmin, related_to=Book)
     """
-    change_list_template = "admin/basecontent/sortable_change_list.html"
+    change_list_template = "admin/basecontent/related_sortable_change_list.html"
     sortablefield = 'position'
 
     def get_ordering(self):
@@ -1202,13 +1305,13 @@ class OrderableRelatedModelAdmin(RelatedModelAdmin):
         relation_lookup = field.field.rel.through.lower()
         return ('%s__order' % relation_lookup, 'asc')
 
-    def changelist_view(self, request, extra_context=None):
-        extra_context = self._update_extra_context(request, extra_context)
+    def changelist_view(self, request, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
         if request.method == 'POST':
             neworder_list = request.POST.get('neworder', None)
             page = request.GET.get('p', 0)
             if neworder_list is None:
-                return super(OrderableRelatedModelAdmin, self).changelist_view(request, extra_context)
+                return super(OrderableRelatedModelAdmin, self).changelist_view(request, extra_context, parent_model_admin, parent_object)
             neworder_list = neworder_list.split(',')
             items = self.queryset(request).filter(id__in=neworder_list)
             for item in items:
@@ -1219,7 +1322,7 @@ class OrderableRelatedModelAdmin(RelatedModelAdmin):
                 setattr(relation, self.sortablefield, neworder)
                 relation.save()
 
-        return super(OrderableRelatedModelAdmin, self).changelist_view(request, extra_context)
+        return super(OrderableRelatedModelAdmin, self).changelist_view(request, extra_context, parent_model_admin, parent_object)
 
     def get_relation_obj(self, through_model, obj):
         """
@@ -1229,12 +1332,35 @@ class OrderableRelatedModelAdmin(RelatedModelAdmin):
         raise NotImplementedError('You have to override this method')
 
 
+class PermissionRelatedAdmin(RelatedModelAdmin, PermissionAdmin):
+    tool_name = 'manage_permissions'
+    tool_label = _('permissions')
+    change_roles_template = 'admin/basecontent/role_permissions.html'
+
+    def changelist_view(self, request, extra_context=None, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
+        extra_context.update({'original': None,
+                              'content': self.basecontent})
+        return self.change_roles_permissions(request, self.basecontent.id, extra_context=extra_context)
+
+
+class AnnouncementAdmin(AnnouncementDefaultAdmin):
+    form = AnnouncementAdminForm
+
+
 def register(site):
     ## register admin models
     site.register(BaseContent, BaseContentViewAdmin)
     site.register(Site, SiteAdmin)
     site.register(ProviderRule)
     site.register(StoredOEmbed)
+    site.register(Announcement, AnnouncementAdmin)
+
+    #default notification
+    site.register(NoticeType, NoticeTypeAdmin)
+    site.register(NoticeSetting, NoticeSettingAdmin)
+    site.register(Notice, NoticeAdmin)
+
     register_related_base(site, BaseContent)
     if settings.USE_GIS:
         register_related_gis(site, BaseContent)
@@ -1242,7 +1368,7 @@ def register(site):
 
 def register_related_base(site, related_to):
     site.register_related(ContactInfo, BaseContentRelatedContactInfoAdmin, related_to=related_to)
-
+    site.register_related(BaseContent, PermissionRelatedAdmin, related_to=related_to)
 
 
 # ----- begin monkey patching -----
