@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -24,7 +24,8 @@ from django.conf import settings
 if settings.USE_GIS:
     from django.contrib.gis.db import models
 else:
-    from django.db import models
+    from django.db import models  # pyflakes:ignore
+from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.core.management import call_command
 from django.core.exceptions import ObjectDoesNotExist, SuspiciousOperation
@@ -38,20 +39,23 @@ from django.utils.html import strip_tags
 from django.utils.text import unescape_entities
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ugettext
-from django.contrib.auth.models import User
 
-from cmsutils.db.fields import AutoSlugField
+from cmsutils.db.fields import AutoSlugField, JSONField
 from cmsutils.signals import post_rebuild_db
 if settings.USE_GIS:
-    from south.introspection_plugins import geodjango
+    from south.introspection_plugins import geodjango  # pyflakes:ignore
 from south.modelsinspector import add_introspection_rules
 from south.signals import pre_migrate, post_migrate
 from stdimage import StdImageField
 from transmeta import TransMeta, get_fallback_fieldname
+from tagging.models import Tag
 from tagging.fields import TagField
 
 from merengue.base.managers import BaseContentManager, WorkflowManager
+from merengue.base.review_tasks import review_to_pending_status
+from merengue.urlresolvers import get_url_default_lang
 from merengue.multimedia.models import BaseMultimedia
+from merengue.utils import is_last_application
 
 
 PRIORITY_CHOICES = (
@@ -73,6 +77,8 @@ COMMENTABLE_CHOICES = (
 
 
 class ContactInfo(models.Model):
+    name = models.CharField(verbose_name=_('name'), max_length=200,
+                            blank=True, null=True)
     contact_email = models.EmailField(verbose_name=_('contact email'),
                                      max_length=200, blank=True, null=True)
     contact_email2 = models.EmailField(verbose_name=_('contact email2'),
@@ -118,7 +124,7 @@ class Base(models.Model):
     __metaclass__ = TransMeta
     name = models.CharField(verbose_name=_('name'), max_length=200, db_index=True)
     slug = models.SlugField(verbose_name=_('slug'), max_length=200, db_index=True, unique=True)
-    plain_description = models.TextField(verbose_name=_('description'),
+    plain_description = models.TextField(verbose_name=_('plain text description'),
                                          null=True, blank=True, editable=False)
     description = models.TextField(verbose_name=_('description'),
                                    null=True, blank=True)
@@ -140,21 +146,24 @@ class Base(models.Model):
     def __unicode__(self):
         return self.name or ugettext('Without name')
 
+    def _prepare_plain_text(self, from_field, to_field):
+        original_text = getattr(self, from_field, None)
+        if original_text:
+            original_text = force_unicode(original_text)
+            text = re.sub('<br[^>]*>', u'\n', original_text)
+            text = unescape_entities(text)
+            text = strip_tags(text)
+            text = text.strip()
+            setattr(self, to_field, text)
+        else:
+            setattr(self, to_field, original_text)
+
     def save(self, *args, **kwargs):
         for lang in settings.LANGUAGES:
             field_name = 'description_%s' % lang[0]
             to_field = 'plain_description_%s' % lang[0]
+            self._prepare_plain_text(field_name, to_field)
 
-            original_text = getattr(self, field_name, None)
-            if original_text:
-                original_text = force_unicode(original_text)
-                text = re.sub('<br[^>]*>', u'\n', original_text)
-                text = unescape_entities(text)
-                text = strip_tags(text)
-                text = text.strip()
-                setattr(self, to_field, text)
-            else:
-                setattr(self, to_field, original_text)
         super(Base, self).save(*args, **kwargs)
 
     @permalink
@@ -177,6 +186,7 @@ if settings.USE_GIS:
                                    null=True, blank=True)
         location = models.ForeignKey(Location, verbose_name=_('location'),
                                      null=True, blank=True, editable=False)
+        location.delete_cascade = False
 
         objects = WorkflowManager()
 
@@ -196,7 +206,7 @@ if settings.USE_GIS:
                 return settings.MEDIA_URL + 'merengue/img/map/default_map_icon.png'
 
         def get_icon_tag(self):
-            return '<img src="%s" title="%s"/>' %(self.get_icon(), self._meta.verbose_name)
+            return '<img src="%s" title="%s"/>' % (self.get_icon(), self._meta.verbose_name)
         get_icon_tag.allow_tags = True
 
         def has_location(self):
@@ -216,6 +226,7 @@ if settings.USE_GIS:
         google_minimap.allow_tags = True
 
         def admin_thumbnail(self):
+            #@FIXME: Duplicate code. multimedia/models.py line 205
             if not self.main_image:
                 return ''
             file_access_failed = False
@@ -251,6 +262,7 @@ class BaseContentMeta(TransMeta):
 
             class Meta:
                 content_view_template = 'myapp/mymodel_view.html'
+                content_view_function = 'myapp.mymodel_view'
     '''
 
     def __new__(cls, name, bases, attrs):
@@ -259,10 +271,16 @@ class BaseContentMeta(TransMeta):
             delattr(attrs['Meta'], 'content_view_template')
         else:
             content_view_template = 'content_view.html'
+        if 'Meta' in attrs and hasattr(attrs['Meta'], 'content_view_function'):
+            content_view_function = attrs['Meta'].content_view_function
+            delattr(attrs['Meta'], 'content_view_function')
+        else:
+            content_view_function = None
 
         new_class = super(BaseContentMeta, cls).__new__(cls, name, bases, attrs)
         if hasattr(new_class, '_meta'):
             new_class._meta.content_view_template = content_view_template
+            new_class._meta.content_view_function = content_view_function
         return new_class
 
 
@@ -284,13 +302,20 @@ class BaseContent(BaseClass):
     # this is handled in admin forms
     user_modification_date = models.DateTimeField(verbose_name=_('modification date'),
                                                   blank=True, null=True, editable=False)
+    cached_plain_text = models.TextField(verbose_name=_('cached plain text'),
+                                   null=True, blank=True, editable=False)
 
     last_editor = models.ForeignKey(User, null=True, blank=True, editable=False,
                                     related_name='last_edited_content')
-    last_editor.delete_cascade=False
+    last_editor.delete_cascade = False
+
+    # permission global
+    adquire_global_permissions = models.BooleanField(_('Adquire global permissions'), default=True)
 
     # tagging info
-    tags = TagField(verbose_name=_('Tags'))
+    tags = TagField(verbose_name=_('Tags'),
+                    help_text=_('Tags will be splitted by spaces or commas if present. To force \
+                                tag names, use double quotes.'))
 
     # meta info
     meta_desc = models.TextField(verbose_name=_('meta description'), null=True, blank=True)
@@ -325,14 +350,47 @@ class BaseContent(BaseClass):
                                     null=True, blank=True,
                                     related_name='contents_owned')
 
+    # structural contents
+    no_changeable = models.BooleanField(default=False,
+                                        editable=False)
+    no_deletable = models.BooleanField(default=False,
+                                       editable=False)
+
+    # block control. cached value for controlling if content has special blocks attached
+    has_related_blocks = models.BooleanField(default=False, editable=False,
+                                             db_index=True)
+
+    # structural fields
+    no_changeable_fields = JSONField(null=True, blank=True,
+                                     editable=False)
+
     objects = BaseContentManager()
 
+    def __init__(self, *args, **kwargs):
+        super(BaseContent, self).__init__(*args, **kwargs)
+        self._original_status = self.status
+
+    def generate_plain_text(self):
+        return strip_tags(self.description)
+
+    def _plain_text(self):
+        if not self.cached_plain_text:
+            for lang in settings.LANGUAGES:
+                from_field = 'description_%s' % lang[0]
+                to_field = 'cached_plain_text_%s' % lang[0]
+                self._prepare_plain_text(from_field, to_field)
+            self.save()
+        return self.cached_plain_text
+
+    plain_text = property(_plain_text)
+
     class Meta:
         verbose_name = _('base content')
         verbose_name_plural = _('base contents')
+        translate = ('cached_plain_text', )
         abstract = False
-        ordering = (get_fallback_fieldname('name'), )
         #content_view_template = 'content_view.html' # default definition by BaseContentMeta metaclass
+        ordering = (get_fallback_fieldname('name'), )
 
     def admin_absolute_url(self):
         return '<a href="%s">%s</a>' % (self.get_admin_absolute_url(), self.name)
@@ -352,13 +410,20 @@ class BaseContent(BaseClass):
 
         tags_field = self._meta.get_field('tags')
         tags_field._save(instance=self)
+        # updating the tags may leave some without related items, so we'll delete them
+        for tag in Tag.objects.filter(items__isnull=True):
+            if hasattr(tag, 'itag'):
+                # itag deletion gets rid of the original tag object
+                tag.itag.delete()
+            else:
+                tag.delete()
 
         # Save thumbnail of main_image in model inherited
         main_image_field = self._meta.get_field('main_image')
         try:
             main_image_field._rename_resize_image(instance=self)
         except OSError:
-            pass # this may fail if the image file does not exist
+            pass  # this may fail if the image file does not exist
 
         if object_update_again:
             non_pks = [f for f in self._meta.local_fields if not f.primary_key]
@@ -368,23 +433,48 @@ class BaseContent(BaseClass):
 
     def get_real_instance(self):
         """ get object child instance """
-        if hasattr(self, '_real_instance'): # try looking in our cache
+        def get_subclasses(cls):
+            subclasses = cls.__subclasses__()
+            result = []
+            for subclass in subclasses:
+                if not subclass._meta.abstract:
+                    result.append(subclass)
+                else:
+                    result += get_subclasses(subclass)
+            return result
+
+        if hasattr(self, '_real_instance'):  # try looking in our cache
             return self._real_instance
-        real_instance = getattr(self, self.class_name, self)
-        self._real_instance = real_instance
-        return real_instance
+        subclasses = get_subclasses(self.__class__)
+        if not subclasses:  # already real_instance
+            real_instance = getattr(self, self.class_name, self)
+            self._real_instance = real_instance
+            return real_instance
+        else:
+            subclasses_names = [cls.__name__.lower() for cls in subclasses]
+            for subcls_name in subclasses_names:
+                if hasattr(self, subcls_name):
+                    return getattr(self, subcls_name, self).get_real_instance()
+            return self
 
     @permalink
     def get_absolute_url(self):
         return ('merengue.base.views.public_link', [self._meta.app_label, self._meta.module_name, self.id])
 
+    @permalink
+    def public_link_without_section(self):
+        return self._public_link_without_section()
+
+    def _public_link_without_section(self):
+        return ('merengue.base.views.public_view', [self._meta.app_label, self._meta.module_name, self.id, self.slug])
+
     @permalink
     def public_link(self):
         section = self.get_main_section()
         if section:
-            return ('content_section_view', [section.slug, self.id, self.slug])
+            return section.get_real_instance()._content_public_link(section, self)
         else:
-            return ('merengue.base.views.public_view', [self._meta.app_label, self._meta.module_name, self.id, self.slug])
+            return self._public_link_without_section()
 
     def link_by_user(self, user):
         """ User dependent link. To override in subclasses, if needed """
@@ -397,12 +487,12 @@ class BaseContent(BaseClass):
     def get_main_section(self):
         """ Get main section of a content """
         try:
-            return self.basesection_set.main()
+            return self.sections.main()
         except ObjectDoesNotExist:
             return None
 
     def calculate_rank(self):
-        return 100.0 # default implementation
+        return 100.0  # default implementation
 
     def recalculate_main_image(self):
         """ main image will be first ordered multimedia relation with class "photo" """
@@ -427,6 +517,38 @@ class BaseContent(BaseClass):
     def is_commentable(self):
         return self.commentable == 'allowed'
 
+    def breadcrumbs_first_item(self):
+        from merengue.pluggable.utils import get_plugin_config
+        plugin_dir = self._meta.app_label
+        plugin_config = get_plugin_config(plugin_dir)
+        if plugin_config is None:
+            return None  # item model is outside plugin... no first item generated
+        url_prefix = plugin_config.url_prefixes[0][0]
+        if isinstance(url_prefix, dict):
+            url_prefix = url_prefix.get(get_url_default_lang(), 'en')
+        plugin_url = "/%s/" % url_prefix
+        return (self._meta.verbose_name_plural, plugin_url)
+
+    def breadcrumbs_last_item(self):
+        return (unicode(self), '')
+
+    def breadcrumbs_items(self):
+        urls = []
+        try:
+            first_item = self.breadcrumbs_first_item()
+            if first_item:
+                urls.append(first_item)
+        except ImportError:
+            urls = []
+        last_item = self.breadcrumbs_last_item()
+        if last_item:
+            urls.append(last_item)
+        return urls
+
+    def breadcrumbs(self):
+        urls = self.breadcrumbs_items()
+        return render_to_string('base/breadcrumbs.html', {'urls': urls})
+
 
 def calculate_class_name(instance):
     instance.class_name = instance._meta.module_name
@@ -440,6 +562,22 @@ def base_content_pre_save_handler(sender, instance, **kwargs):
 signals.pre_save.connect(base_content_pre_save_handler)
 
 
+def notify_status_changes(sender, instance, **kwargs):
+    if isinstance(instance, BaseContent):
+        if not hasattr(instance, '_original_status'):
+            return
+        if instance._original_status == instance.status:
+            return
+        if instance.status == 'pending':
+            review_to_pending_status(instance, instance._original_status)
+        # Change cached original status so if we have
+        # multiple saves we don't generate multiple
+        # review tasks
+        instance._original_status = instance.status
+
+signals.post_save.connect(notify_status_changes)
+
+
 class MultimediaRelation(models.Model):
     content = models.ForeignKey(BaseContent, verbose_name=_('content'))
     multimedia = models.ForeignKey(BaseMultimedia,
@@ -477,7 +615,7 @@ class MultimediaRelation(models.Model):
 
     def _get_human_order(self):
         if self.order != None:
-            return self.order+1
+            return self.order + 1
         return ''
     _get_human_order.short_description = _('Human Order')
     human_order = property(_get_human_order)
@@ -497,7 +635,7 @@ class MultimediaRelation(models.Model):
     def get_next_multimedia(self):
         try:
             next = MultimediaRelation.objects.get(content=self.content,
-                                                  order=self.order+1)
+                                                  order=self.order + 1)
             return next
         except MultimediaRelation.DoesNotExist:
             return self
@@ -505,7 +643,7 @@ class MultimediaRelation(models.Model):
     def get_previous_multimedia(self):
         try:
             previous = MultimediaRelation.objects.get(content=self.content,
-                                                      order=self.order-1)
+                                                      order=self.order - 1)
             return previous
         except MultimediaRelation.DoesNotExist:
             return self
@@ -569,7 +707,7 @@ def _collect_sub_objects(self, seen_objs, parent=None, nullable=False):
         if seen_objs.add(self.__class__, pk_val, self,
                          parent, nullable):
             return
-    else: # For Django 1.1.2 or newer
+    else:  # For Django 1.1.2 or newer
         if seen_objs.add(self.__class__, pk_val, self,
                          type(parent), parent, nullable):
             return
@@ -680,10 +818,12 @@ def handle_pre_migrate(sender, **kwargs):
 def handle_post_migrate(sender, **kwargs):
     from merengue.pluggable import enable_active_plugins
     global post_save_receivers, cache_backend
+    app = kwargs['app']
+    if is_last_application(app):
+        enable_active_plugins()
     # site fixtures loading after migration
-    enable_active_plugins()
     for app_name, fixtures in getattr(settings, 'SITE_FIXTURES', {}).items():
-        if app_name == kwargs['app']: # only migrate
+        if app_name == app:  # only migrate
             for fixture in fixtures:
                 call_command('loaddata', fixture, verbosity=1)
     # will set again saved receivers and cache backend
