@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -16,6 +16,7 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 from django import forms
+from django.conf import settings
 from django.conf.urls.defaults import patterns
 from django.contrib import admin
 from django.contrib.admin.widgets import FilteredSelectMultiple
@@ -34,10 +35,13 @@ from merengue.perms import ANONYMOUS_ROLE_SLUG
 from merengue.perms.models import ObjectPermission, Permission, PrincipalRoleRelation, Role
 from merengue.perms.forms import UserChangeForm, GroupForm, PrincipalRoleRelationForm
 from merengue.perms.utils import add_role, remove_role, can_manage_user
+from merengue.base.widgets import ReadOnlyWidget
 
 
 class PermissionAdmin(admin.ModelAdmin):
 
+    change_roles_template = 'admin/perms/objectpermission/role_permissions.html'
+
     def has_change_permission(self, request, obj=None):
         """
         Overrides Django admin behaviour to add ownership based access control
@@ -81,17 +85,21 @@ class PermissionAdmin(admin.ModelAdmin):
 
     def _update_role_users(self, request, obj):
         selected = request.POST.getlist('selected_user')
+        users_to_delete = request.POST.getlist('delete_user_roles')
+        for uppr in PrincipalRoleRelation.objects.filter(content=obj, user__in=users_to_delete):
+            uppr.delete()
         for ppr in PrincipalRoleRelation.objects.filter(content=obj, user__isnull=False):
             role_user = "%s_%s" % (ppr.role.id, ppr.user.id)
             if role_user not in selected:
                 ppr.delete()
         for role_user in selected:
             role_id, user_id = role_user.split('_')
-            role = Role.objects.get(id=role_id)
-            user = User.objects.get(id=user_id)
-            ppr, created = PrincipalRoleRelation.objects.get_or_create(role=role,
-                                                                       user=user,
-                                                                       content=obj)
+            if user_id not in users_to_delete:
+                role = Role.objects.get(id=role_id)
+                user = User.objects.get(id=user_id)
+                ppr, created = PrincipalRoleRelation.objects.get_or_create(role=role,
+                                                                           user=user,
+                                                                           content=obj)
         msg = _('The role users were changed successfully.')
         if '_users_continue' in request.POST:
             url_redirect = '.'
@@ -101,17 +109,21 @@ class PermissionAdmin(admin.ModelAdmin):
 
     def _update_role_groups(self, request, obj):
         selected = request.POST.getlist('selected_group')
+        groups_to_delete = request.POST.getlist('delete_group_roles')
+        for gppr in PrincipalRoleRelation.objects.filter(content=obj, group__in=groups_to_delete):
+            gppr.delete()
         for ppr in PrincipalRoleRelation.objects.filter(content=obj, group__isnull=False):
             role_group = "%s_%s" % (ppr.role.id, ppr.group.id)
             if role_group not in selected:
                 ppr.delete()
         for role_group in selected:
             role_id, group_id = role_group.split('_')
-            role = Role.objects.get(id=role_id)
-            group = Group.objects.get(id=group_id)
-            ppr, created = PrincipalRoleRelation.objects.get_or_create(role=role,
-                                                                       group=group,
-                                                                       content=obj)
+            if group_id not in groups_to_delete:
+                role = Role.objects.get(id=role_id)
+                group = Group.objects.get(id=group_id)
+                ppr, created = PrincipalRoleRelation.objects.get_or_create(role=role,
+                                                                           group=group,
+                                                                           content=obj)
         msg = _('The role groups were changed successfully.')
         if '_groups_continue' in request.POST:
             url_redirect = '.'
@@ -136,8 +148,11 @@ class PermissionAdmin(admin.ModelAdmin):
                 msg, url_redirect = self._update_role_users(request, obj)
             elif '_groups_continue' in request.POST or '_groups_save' in request.POST:
                 msg, url_redirect = self._update_role_groups(request, obj)
+            # set if object should adquire global permissions
+            obj.adquire_global_permissions = request.POST.get('adquire_global_permissions', False)
+            obj.save()
             if msg and url_redirect:
-                return self.response_change(request, msg, url_redirect=url_redirect)
+                return self.response_change_permissions(request, msg, url_redirect=url_redirect)
 
         roles = Role.objects.all()
         role_permissions = {}
@@ -187,13 +202,14 @@ class PermissionAdmin(admin.ModelAdmin):
                    'group_roles': group_roles,
                    'anonymous_role_slug': ANONYMOUS_ROLE_SLUG, }
 
-
-        template = 'admin/perms/objectpermission/role_permissions.html'
+        extra_context = extra_context or {}
+        context.update(extra_context)
+        template = self.change_roles_template
         return render_to_response(template,
                                   context,
                                   context_instance=RequestContext(request))
 
-    def response_change(self, request, *args, **kwargs):
+    def response_change_permissions(self, request, *args, **kwargs):
         """
         Determines the HttpResponse for the change_view stage.
         """
@@ -249,8 +265,7 @@ class ObjectPermissionAdmin(PermissionAdmin):
                 perm = Permission.objects.get(id=perm_id)
                 op, created = ObjectPermission.objects.get_or_create(role=role, permission=perm, content=None)
 
-            return self.response_change(request)
-
+            return self.response_change_permissions(request)
 
         roles = Role.objects.all()
         permissions = {}
@@ -260,7 +275,6 @@ class ObjectPermissionAdmin(PermissionAdmin):
             for role in roles:
                 permissions[perm].append((role, perm.objectpermission_set.filter(role=role) and True or False))
 
-
         context = {'admin_site': admin_site.name,
                    'change': True,
                    'is_popup': False,
@@ -277,7 +291,6 @@ class ObjectPermissionAdmin(PermissionAdmin):
                    'role_permissions': permissions,
                    'roles': roles}
 
-
         template = 'admin/perms/objectpermission/role_permissions.html'
         return render_to_response(template,
                                   context,
@@ -294,6 +307,22 @@ class RoleAdmin(admin.ModelAdmin):
         """
         return can_manage_user(request.user)
 
+    def has_delete_permission(self, request, obj=None):
+        if obj and obj.name in settings.STATIC_ROLES:
+            return False
+        elif obj:
+            return True
+        else:
+            if request.method == 'POST' and \
+               request.POST.get('action', None) == u'delete_selected':
+                selected_objs = [Role.objects.get(id=int(key))
+                                 for key in request.POST.getlist('_selected_action')]
+                for sel_obj in selected_objs:
+                    if not self.has_delete_permission(request, sel_obj):
+                        return False
+                return True
+        return False
+
     def save_model(self, request, obj, form, change):
         super(RoleAdmin, self).save_model(request, obj, form, change)
         selected = request.POST.getlist('selected_perm')
@@ -304,6 +333,17 @@ class RoleAdmin(admin.ModelAdmin):
             if created:
                 op.save()
 
+    def get_form(self, request, obj=None, **kwargs):
+        form = super(RoleAdmin, self).get_form(request, obj, **kwargs)
+        if obj and obj.name in settings.STATIC_ROLES:
+            form.base_fields['name'].widget = ReadOnlyWidget(
+                getattr(obj, 'name', ''), obj.name)
+            form.base_fields['name'].required = False
+            form.base_fields['slug'].widget = ReadOnlyWidget(
+                getattr(obj, 'slug', ''), obj.slug)
+            form.base_fields['slug'].required = False
+        return form
+
     def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
         permissions = []
         if request.method == 'POST':
