@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -16,49 +16,75 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 # -*- coding: utf-8 -*-
+
 import os
 import sys
 try:
     from cStringIO import StringIO
 except ImportError:
-    from StringIO import StringIO
-
+    from StringIO import StringIO  # pyflakes:ignore
 
 from django import templatetags
 from django.conf import settings
 from django.conf.urls.defaults import include, url
 from django.contrib.admin.sites import NotRegistered
-from django.core.exceptions import ImproperlyConfigured, FieldError
+from django.core.exceptions import ImproperlyConfigured, FieldError, MiddlewareNotUsed
+from django.core.management import call_command
 from django.core.management.color import no_style
 from django.core.management.sql import sql_all
 from django.core.management.validation import get_validation_errors
 from django.core import urlresolvers
 from django.db import connection, transaction
 from django.db.models import get_models
-from django.db.models.loading import load_app
+from django.db.models.loading import load_app, cache
 from django.utils.importlib import import_module
 
+from south import migration
+from south.exceptions import NoMigrations
+
 from merengue import registry
 from merengue.base.adminsite import site
-from merengue.registry.items import (NotRegistered as NotRegisteredItem,
-                                     AlreadyRegistered as AlreadyRegisteredItem)
-from merengue.section.models import Section
+from merengue.registry.items import (NotRegistered as NotRegisteredItem)
+from merengue.section.models import BaseSection
+from merengue.section.middleware import register_section_prefix, unregister_section_prefix
 from merengue.perms.utils import register_permission, unregister_permission
+from merengue.pluggable.exceptions import BrokenPlugin
+from merengue.pluggable.models import RegisteredPlugin
 
 
-def install_plugin(instance, app_name):
-    app_mod = load_app(app_name)
+# ----- internal attributes and methods -----
+
+
+_plugin_middlewares_cache = {
+    'loaded_middlewares': [],
+    'request_middleware': [],
+    'view_middleware': [],
+    'response_middleware': [],
+    'exception_middleware': [],
+}
+
+
+# ----- public methods -----
+
+
+def install_plugin(registered_plugin):
+    app_name = registered_plugin.directory_name
+    plugin_name = get_plugin_module_name(app_name)
     # Needed update installed apps in order
     # to get SQL command from merengue.pluggable
-    add_to_installed_apps(app_name)
-    if app_mod and not are_installed_models(app_mod):
-        install_models(app_mod)
-        # Force instance saving after connection closes.
-        instance.save()
-    if instance.active:
-        enable_plugin(app_name)
+    add_to_installed_apps(plugin_name)
+    if load_app(plugin_name) and not are_installed_models(app_name):
+        install_models(app_name)
+        reload_models_cache()
+        # Force registered_plugin saving after connection closes.
+        registered_plugin.save()
+    if registered_plugin.active:
+        enable_plugin(plugin_name)
+        # Doing extra custom installation implemented in each plugin
+        plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+        plugin.post_install()
     else:
-        disable_plugin(app_name)
+        disable_plugin(plugin_name)
 
 
 def get_plugins_dir():
@@ -67,6 +93,13 @@ def get_plugins_dir():
     return plugins_dir
 
 
+def get_plugin_directories():
+    """ get all plugins directories """
+    plugins_path = os.path.join(settings.BASEDIR, get_plugins_dir())
+    return [d for d in os.listdir(plugins_path) if os.path.isdir(os.path.join(plugins_path, d)) and
+                                                   not d.startswith('.')]
+
+
 def get_plugin_module_name(plugin_dir):
     return '%s.%s' % (get_plugins_dir(), plugin_dir)
 
@@ -79,16 +112,38 @@ def get_plugin_config(plugin_dir, prepend_plugins_dir=True):
             plugin_modname = '%s.config' % plugin_dir
         return getattr(import_module(plugin_modname), 'PluginConfig')
     except (ImportError, TypeError, ValueError):
+        # only will raise exception in debug mode
+        # this prevents broke the whole portal when registering a broken plugin
+        if settings.DEBUG:
+            raise
         return None
 
 
+def get_plugin(plugin_dir, prepend_plugins_dir=True):
+    plugin_config = get_plugin_config(plugin_dir, prepend_plugins_dir)
+    reg_plugin = RegisteredPlugin.objects.by_item_class(plugin_config).get()
+    return reg_plugin.get_registry_item()
+
+
 def validate_plugin(plugin_config):
     if not plugin_config.name:
         raise ImproperlyConfigured('Plugin %s must have a defined "name" attribute' % plugin_config)
 
 
-def are_installed_models(plugin_mod):
-    plugin_models = get_models(plugin_mod)
+def have_south(app_name):
+    try:
+        migrations = migration.Migrations(app_name)  # pyflakes:ignore
+    except NoMigrations:
+        return False
+    else:
+        return True
+
+
+def are_installed_models(app_name):
+    if have_south(app_name):
+        return False
+    app_module = load_app(get_plugin_module_name(app_name))
+    plugin_models = get_models(app_module)
     tables = connection.introspection.table_names()
     seen_models = connection.introspection.installed_models(tables)
     for plugin_model in plugin_models:
@@ -97,13 +152,17 @@ def are_installed_models(plugin_mod):
     return True
 
 
-def install_models(plugin_mod):
-    style = no_style()
-    cursor = connection.cursor()
-    sql_commands = sql_all(plugin_mod, style)
-    for sql_command in sql_commands:
-        cursor.execute(sql_command)
-    transaction.commit_unless_managed()
+def install_models(app_name):
+    app_module = load_app(get_plugin_module_name(app_name))
+    if have_south(app_name):
+        call_command('migrate', app=app_name)
+    else:
+        style = no_style()
+        cursor = connection.cursor()
+        sql_commands = sql_all(app_module, style)
+        for sql_command in sql_commands:
+            cursor.execute(sql_command)
+    transaction.commit()
 
 
 def add_to_installed_apps(plugin_name):
@@ -127,10 +186,10 @@ def remove_from_installed_apps(plugin_name):
 
 
 def find_plugin_urls(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    for url_prefix, urlconf in plugin_config.url_prefixes:
+    for url_prefix, urlconf in plugin.get_url_prefixes():
         plugin_url_re = r'^%s/' % url_prefix
         try:
             import_module(urlconf)
@@ -146,9 +205,10 @@ def find_plugin_urls(plugin_name):
         yield (index, plugin_url)
 
 
-def is_plugin_broken(plugin_name):
-    """ Returns if plugin is broken (not exist in file system) """
-    if get_plugin_config(plugin_name, prepend_plugins_dir=False):
+def check_plugin_broken(plugin_name):
+    """ Check if plugin is broken (i.e. not exist in file system) and raises an exception """
+    try:
+        get_plugin_config(plugin_name, prepend_plugins_dir=False)
         try:
             # try to import plugin modules (if exists) to validate those models
             models_modname = '%s.models' % plugin_name
@@ -161,32 +221,32 @@ def is_plugin_broken(plugin_name):
         except ImportError:
             # usually means models module does not exists. Don't worry about that
             pass
-        except (TypeError, FieldError, SyntaxError): # some validation error when importing models
-            return True
-        return False
-    else:
-        return True
+        except (TypeError, FieldError, SyntaxError):  # some validation error when importing models
+            raise BrokenPlugin(plugin_name, *sys.exc_info())
+    except Exception:
+        raise BrokenPlugin(plugin_name, *sys.exc_info())
 
 
 def enable_plugin(plugin_name, register=True):
     from merengue.base.admin import register_app
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
-        return
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
     add_to_installed_apps(plugin_name)
     if register:
         register_app(plugin_name)
         register_plugin_actions(plugin_name)
         register_plugin_blocks(plugin_name)
+        register_plugin_toolbar(plugin_name)
+        register_plugin_middlewares(plugin_name)
         register_plugin_viewlets(plugin_name)
         register_plugin_templatetags(plugin_name)
         register_plugin_post_actions(plugin_name)
         register_plugin_section_models(plugin_name)
         register_plugin_in_plugin_admin_site(plugin_name)
         register_plugin_perms(plugin_name)
+        register_plugin_section_prefixes(plugin_name)
     register_plugin_urls(plugin_name)
     # activate plugin in DB
-    registered_plugin = plugin_config.get_registered_item()
+    registered_plugin = plugin.get_registered_item()
     registered_plugin.activate()
     # app_directories template loader loads app_template_dirs in
     # compile time, so we have to load it again.
@@ -203,15 +263,20 @@ def disable_plugin(plugin_name, unregister=True):
             pass
         unregister_plugin_actions(plugin_name)
         unregister_plugin_blocks(plugin_name)
+        unregister_plugin_toolbar(plugin_name)
+        unregister_plugin_middlewares(plugin_name)
         unregister_plugin_viewlets(plugin_name)
         unregister_plugin_templatetags(plugin_name)
         unregister_plugin_section_models(plugin_name)
         unregister_plugin_in_plugin_admin_site(plugin_name)
         unregister_plugin_perms(plugin_name)
+        unregister_plugin_section_prefixes(plugin_name)
     unregister_plugin_urls(plugin_name)
     # app_directories template loader loads app_template_dirs in
     # compile time, so we have to load it again.
     reload_app_directories_template_loader()
+    # clear models cache to populate new models
+    reload_models_cache()
 
 
 def register_plugin_urls(plugin_name):
@@ -259,128 +324,189 @@ def unregister_plugin_templatetags(plugin_name):
 
 
 def register_items(item_list):
-    try:
-        for item_class in item_list:
+    for item_class in item_list:
+        if not registry.have_registered_items(item_class):
             registry.register(item_class, activate=True)
-    except AlreadyRegisteredItem:
-        pass
 
 
 def unregister_items(item_list):
     try:
         for item_class in item_list:
-            registry.unregister(item_class)
+            registry.unregister_all(item_class)
     except NotRegisteredItem:
         pass
 
 
 def register_plugin_actions(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    register_items(plugin_config.get_actions())
+    register_items(plugin.get_actions())
 
 
 def unregister_plugin_actions(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    unregister_items(plugin_config.get_actions())
+    unregister_items(plugin.get_actions())
 
 
 def register_plugin_blocks(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    register_items(plugin_config.get_blocks())
+    register_items(plugin.get_blocks())
 
 
 def unregister_plugin_blocks(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    unregister_items(plugin_config.get_blocks())
+    unregister_items(plugin.get_blocks())
 
 
 def register_plugin_viewlets(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    register_items(plugin_config.get_viewlets())
+    register_items(plugin.get_viewlets())
 
 
 def unregister_plugin_viewlets(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    unregister_items(plugin_config.get_viewlets())
+    unregister_items(plugin.get_viewlets())
+
+
+def register_plugin_toolbar(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    register_items(plugin.get_toolbar_panels())
+
+
+def unregister_plugin_toolbar(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    unregister_items(plugin.get_toolbar_panels())
 
 
 def register_plugin_post_actions(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    plugin_config.post_actions()
+    plugin.post_actions()
 
 
 def register_plugin_section_models(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
-    register_plugin_section_models_in_admin_site(plugin_config, plugin_name, site)
-
-
-def register_plugin_section_models_in_admin_site(plugin_config, plugin_name, admin_site):
-    if not admin_site:
-        return
-    for model, admin_model in plugin_config.section_models():
-        site_related = admin_site.register_related(model, admin_model, related_to=Section)
-        plugin_config.section_register_hook(site_related, model)
-
-
-def register_plugin_in_plugin_admin_site(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
-        return
-    model_admins = plugin_config.get_model_admins()
-    if not model_admins:
-        return
-    plugin_site = site.register_plugin_site(plugin_name)
-    for model, admin_model in plugin_config.get_model_admins():
-        plugin_site.register(model, admin_model)
-
-
-def register_plugin_perms(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
-        return
-    for perm in plugin_config.get_perms():
-        register_permission(*perm)
-
-
-def unregister_plugin_perms(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
-        return
-    for perm in plugin_config.get_perms():
-        unregister_permission(perm[1])
+    register_plugin_section_models_in_admin_site(plugin, plugin_name, site)
 
 
 def unregister_plugin_section_models(plugin_name):
     pass
 
 
+def register_plugin_section_models_in_admin_site(plugin, plugin_name, admin_site):
+    if not admin_site:
+        return
+    for model, admin_model in plugin.section_models():
+        site_related = admin_site.register_related(model, admin_model, related_to=BaseSection)
+        plugin.section_register_hook(site_related, model)
+
+
+def register_plugin_in_plugin_admin_site(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    model_admins = plugin.get_model_admins()
+    if not model_admins:
+        return
+    plugin_site = site.register_plugin_site(plugin_name)
+    for model, admin_model in plugin.get_model_admins():
+        plugin_site.register(model, admin_model)
+
+
 def unregister_plugin_in_plugin_admin_site(plugin_name):
-    plugin_config = get_plugin_config(plugin_name, prepend_plugins_dir=False)
-    if not plugin_config:
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
         return
     else:
-        plugin = plugin_config.get_registered_item()
+        plugin = plugin.get_registered_item()
         if not plugin.installed:
             return
     site.unregister_plugin_site(plugin_name)
 
 
+def register_plugin_perms(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for perm in plugin.get_perms():
+        register_permission(*perm)
+
+
+def unregister_plugin_perms(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for perm in plugin.get_perms():
+        unregister_permission(perm[1])
+
+
+def register_plugin_middlewares(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for middleware in plugin.get_middlewares():
+        register_middleware(middleware)
+
+
+def unregister_plugin_middlewares(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for middleware in plugin.get_middlewares():
+        unregister_middleware(middleware)
+
+
+def register_plugin_section_prefixes(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for section_prefix in plugin.get_section_prefixes():
+        register_section_prefix(section_prefix)
+
+
+def unregister_plugin_section_prefixes(plugin_name):
+    plugin = get_plugin(plugin_name, prepend_plugins_dir=False)
+    if not plugin:
+        return
+    for section_prefix in plugin.get_section_prefixes():
+        unregister_section_prefix(section_prefix)
+
+
+def reload_models_cache():
+    """
+    Reload Django internal cache for all models in installed apps.
+    This includes model field mapping, related fields in _meta object, etc.
+    """
+    cache.loaded = False
+    cache.handled = {}
+    cache.app_store.clear()
+    cache._populate()
+    for model in get_models():
+        opts = model._meta
+        if hasattr(opts, '_related_many_to_many_cache'):
+            # we remove related m2m fields cache for all models
+            del opts._related_many_to_many_cache
+        opts.init_name_map()
+
+
 def reload_app_directories_template_loader():
     """
     This function reloads the template directories with new apps installed.
@@ -437,5 +563,70 @@ def has_required_dependencies(plugin):
     return True
 
 
+def register_middleware(middleware_path):
+    """
+    Load middleware into plugin middleware registry
+    """
+    global _plugin_middlewares_cache
+    if middleware_path in _plugin_middlewares_cache['loaded_middlewares']:
+        return  # already registered
+    try:
+        dot = middleware_path.rindex('.')
+    except ValueError:
+        raise ImproperlyConfigured('%s isn\'t a middleware module' % middleware_path)
+    mw_module, mw_classname = middleware_path[:dot], middleware_path[dot + 1:]
+    try:
+        mod = import_module(mw_module)
+    except ImportError, e:
+        raise ImproperlyConfigured('Error importing middleware %s: "%s"' % (mw_module, e))
+    try:
+        mw_class = getattr(mod, mw_classname)
+    except AttributeError:
+        raise ImproperlyConfigured('Middleware module "%s" does not define a "%s" class' % (mw_module, mw_classname))
+
+    try:
+        mw_instance = mw_class()
+    except MiddlewareNotUsed:
+        return
+
+    if hasattr(mw_instance, 'process_request'):
+        _plugin_middlewares_cache['request_middleware'].append(
+            (middleware_path, mw_instance.process_request),
+        )
+    if hasattr(mw_instance, 'process_view'):
+        _plugin_middlewares_cache['view_middleware'].append(
+            (middleware_path, mw_instance.process_view),
+        )
+    if hasattr(mw_instance, 'process_response'):
+        _plugin_middlewares_cache['response_middleware'].insert(
+            0, (middleware_path, mw_instance.process_response),
+        )
+    if hasattr(mw_instance, 'process_exception'):
+        _plugin_middlewares_cache['exception_middleware'].insert(
+            0, (middleware_path, mw_instance.process_exception),
+        )
+    _plugin_middlewares_cache['loaded_middlewares'].append(middleware_path)
+
+
+def unregister_middleware(middleware_path):
+    """
+    Unload middleware into plugin middleware registry
+    """
+    global _plugin_middlewares_cache
+    if middleware_path not in _plugin_middlewares_cache['loaded_middlewares']:
+        return  # not registered
+    for midd_type in ('request_middleware', 'request_middleware',
+                      'response_middleware', 'exception_middleware', ):
+        for path, method in _plugin_middlewares_cache[midd_type]:
+            if middleware_path == path:
+                _plugin_middlewares_cache[midd_type].remove((path, method))
+    _plugin_middlewares_cache['loaded_middlewares'].remove(middleware_path)
+
+
+def get_plugins_middleware_methods(midd_type):
+    global _plugin_middlewares_cache
+    return (t[1] for t in _plugin_middlewares_cache[midd_type])
+
+
 from merengue.section.admin import register as register_section
 register_section(site)
