@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -15,9 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
-import md5
-
-from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse
@@ -25,17 +22,15 @@ from django.db import models
 from django.db.models import permalink
 from django.db.models.signals import post_save
 from django.template import defaultfilters
+from django.template.loader import render_to_string
 from django.utils.translation import ugettext_lazy as _
 
 import mptt
 
-from cmsutils.adminfilters import QueryStringManager
-
 from merengue.base.managers import WorkflowManager
-from merengue.base.models import Base, BaseContent
+from merengue.base.models import BaseContent
 from merengue.section.managers import SectionManager
 from merengue.viewlet.models import RegisteredViewlet
-from searchform.registry import search_form_registry
 from transmeta import TransMeta, get_fallback_fieldname
 
 
@@ -121,15 +116,26 @@ class Menu(models.Model):
             self.update_url()
         return self.url
 
+    @permalink
+    def menu_public_link_with_out_section(self, ancestors_path):
+        return self._menu_public_link_without_section(self, ancestors_path)
+
+    def _menu_public_link_without_section(self, ancestors_path):
+        return ('menu_view', (ancestors_path, self.slug))
+
+    @permalink
+    def public_link(self):
+        menus_ancestors = [menu.slug for menu in self.get_ancestors()][1:]  # first is a dummy root menu and we discard it
+        ancestors_path = menus_ancestors and '/%s' % '/'.join(menus_ancestors) or ''
+        section = self.get_section()
+        if section:
+            return section.get_real_instance()._menu_public_link(ancestors_path, self)
+        else:
+            return self._menu_public_link_without_section(ancestors_path)
+
     def update_url(self, commit=True):
         try:
-            menus_ancestors = [menu.slug for menu in self.get_ancestors()][1:] # first is a dummy root menu and we discard it
-            ancestors_path = menus_ancestors and '/%s' % '/'.join(menus_ancestors) or ''
-            section = self.get_section()
-            if section:
-                self.url = reverse('menu_section_view', args=(section.slug, ancestors_path, self.slug))
-            else:
-                self.url = reverse('menu_view', args=(ancestors_path, self.slug))
+            self.url = self.public_link()
         except BaseLink.DoesNotExist:
             self.url = ''
         if commit:
@@ -201,6 +207,19 @@ class AbsoluteLink(BaseLink):
         help_text=_('The absolute urls have to write complety: Protocol, domain, query'),
     )
 
+    target_blank = models.BooleanField(
+        verbose_name=_('open in new window'),
+        blank=True,
+        default=False,
+    )
+
+    link_class = models.CharField(
+        verbose_name=_('css class for this link'),
+        max_length=200,
+        blank=True,
+        null=True,
+    )
+
     def get_absolute_url(self):
         return self.url
 
@@ -232,9 +251,7 @@ class ViewletLink(BaseLink):
         return True
 
 
-class BaseSection(Base, RealInstanceMixin):
-
-    __metaclass__ = TransMeta
+class BaseSection(BaseContent):
 
     order = models.IntegerField(
         _('order'),
@@ -254,8 +271,8 @@ class BaseSection(Base, RealInstanceMixin):
 
     main_content = models.ForeignKey(
         BaseContent,
-        null = True,
-        blank = True,
+        null=True,
+        blank=True,
         verbose_name=_('main content'),
         help_text=_('content selected here will be shown when entering the section'),
         related_name='section_main_content',
@@ -264,6 +281,7 @@ class BaseSection(Base, RealInstanceMixin):
 
     related_content = models.ManyToManyField(
         BaseContent,
+        related_name='sections',
         verbose_name=_('related contents'),
         editable=False,
         through='SectionRelatedContent',
@@ -271,9 +289,9 @@ class BaseSection(Base, RealInstanceMixin):
 
     customstyle = models.ForeignKey(
         'CustomStyle',
-        null = True,
-        blank = True,
-        editable = False,
+        null=True,
+        blank=True,
+        editable=False,
         verbose_name=_('custom style'),
     )
     customstyle.delete_cascade = False
@@ -281,41 +299,71 @@ class BaseSection(Base, RealInstanceMixin):
     objects = SectionManager()
 
     class Meta:
-        abstract = False
         ordering = ('order', )
+        verbose_name = _('section')
 
     def __unicode__(self):
         return unicode(self.name)
 
-    @property
-    def app_name(self):
-        return self.real_instance.app_name
-
     @property
     def check_attributes(self):
         return [subcl._meta.module_name for subcl in self.__class__.__subclasses__()]
 
-    def get_absolute_url(self):
-        return strip_section_prefix(self.real_instance.get_absolute_url())
+    def public_link(self):
+        url = super(BaseSection, self).public_link()
+        return strip_section_prefix(url)
+
+    def _public_link_without_section(self):
+        return ('section_view', [self.slug])
 
     def get_breadcrumbs(self):
         return [(self.get_absolute_url(), unicode(self))]
 
-    @permalink
-    def get_agenda_url(self):
-        return ("%s_agenda_view" %(self.app_name), [])
-
     @permalink
     def get_admin_absolute_url(self):
         content_type = ContentType.objects.get_for_model(self)
         return ('merengue.base.views.admin_link', [content_type.id, self.id, ''])
 
+    @permalink
+    def content_public_link(self, section, content):
+        return self._content_public_link(section, content)
+
+    def _content_public_link(self, section, content):
+        return ('content_section_view', [section.slug, content.id, content.slug])
+
+    @permalink
+    def document_public_link(self, section, content):
+        return self._document_public_link(section, content)
+
+    def _document_public_link(self, section, content):
+        return ('document_section_view', [section.slug, content.id, content.slug])
+
+    @permalink
+    def menu_public_link(self, ancestors_path, menu):
+        return self._menu_public_link(ancestors_path, menu)
+
+    def _menu_public_link(self, ancestors_path, menu):
+        return ('menu_section_view', (self.slug, ancestors_path, menu.slug))
+
     def has_custom_style(self):
         return bool(self.customstyle)
 
+    def breadcrumbs_items(self):
+        return [(unicode(self), self.get_absolute_url())]
+
+    def breadcrumbs(self, content=None):
+        urls = self.breadcrumbs_items()
+        if content:
+            urls.append((unicode(content), ''))
+        return render_to_string('section/breadcrumbs.html', {'urls': urls})
+
+    def url_in_section(self, url):
+        return url
+
 
 def strip_section_prefix(link):
-    return link.replace('%s/sections/' % settings.MERENGUE_URLS_PREFIX, '')
+    section_index_url = reverse('section_index')[:-1]
+    return link.replace('%s' % section_index_url, '')
 
 
 def sections_permalink(func):
@@ -328,7 +376,7 @@ def sections_permalink(func):
 
 
 class SectionRelatedContent(models.Model):
-    basesection = models.ForeignKey(BaseSection)
+    basesection = models.ForeignKey(BaseSection, related_name='sectionrelatedcontent')
     basecontent = models.ForeignKey(BaseContent)
     order = models.PositiveIntegerField(default=0)
 
@@ -336,35 +384,8 @@ class SectionRelatedContent(models.Model):
         db_table = 'section_basesection_related_content'
 
 
-class Section(BaseSection):
-    objects = SectionManager()
-
-    @property
-    def app_name(self):
-        return ''
-
-    @sections_permalink
-    def get_absolute_url(self):
-        return ('section_view', (self.slug, ))
-
-
 class Document(BaseContent):
 
-    search_form = models.CharField(
-        verbose_name=_('search form'),
-        max_length=200,
-        blank=True,
-        null=True,
-        editable=False, # until search form feature was completed
-    )
-
-    search_form_filters = models.TextField(
-        verbose_name=_('searcher options'),
-        blank=True,
-        null=True,
-        editable=False, # until search form feature was completed
-    )
-
     permanent = models.BooleanField(verbose_name=_('permanent'),
                                     help_text=_('make this document not erasable and its slug ummutable'),
                                     editable=False,
@@ -381,69 +402,12 @@ class Document(BaseContent):
     def __unicode__(self):
         return unicode(self.name)
 
-    def get_search_form(self):
-        if self.search_form:
-            form_class = search_form_registry.get_form_class(self.search_form)
-            qsm = QueryStringManager(None)
-            filters = dict([(f, []) for f in self.get_search_form_filters()])
-            for filter in filters:
-                options = self.get_search_form_filters_options(filter)
-                filters[filter] = options
-                if search_form_registry.can_select_multiple_options(
-                    self.search_form,
-                    filter,
-                    ):
-                    qsm.filters.setlist(filter + '__in', options)
-                else:
-                    qsm.filters[filter + '__exact'] = options[0]
-
-            form = form_class(query_string_manager=qsm)
-            for filter, options in filters.items():
-                field = form.fields[filter]
-                field.filters['id__in'] = options
-
-            return form
-
-    def get_search_form_filters(self):
-        if not self.search_form_filters:
-            return []
-        result = []
-        for filter_and_options in self.search_form_filters.split('\n'):
-            if filter_and_options:
-                result.append(filter_and_options.split(':')[0].strip())
-        return result
-
-    def get_search_form_filters_options(self, filter_name):
-        if not self.search_form_filters:
-            return []
-        for filter_and_options in self.search_form_filters.split('\n'):
-            if filter_and_options:
-                (name, options) = filter_and_options.split(':')
-                name=name.strip()
-                if name == filter_name:
-                    option_list = []
-                    for option in options.split(','):
-                        soption = option.strip()
-                        if soption.isdigit():
-                            option_list.append(int(soption))
-                        else:
-                            option_list.append(soption)
-                    return option_list
-        return []
-
-    def get_cache_key(self):
-        """ returns a cache key that identifies this document with search form
-            configuration set """
-        cache_key = ''
-        if self.related_section:
-            cache_key += self.related_section.slug
-        cache_key += self.search_form
-        cache_key += md5.md5(self.search_form_filters or '').hexdigest()
-        return cache_key
-
     @permalink
     def public_link(self):
-        return ('document_section_view', [self.get_main_section().slug, self.id, self.slug])
+        section = self.get_main_section()
+        if not section:
+            return self._public_link_without_section()
+        return section.get_real_instance()._document_public_link(section, self)
 
 
 class DocumentSection(models.Model):
@@ -453,7 +417,7 @@ class DocumentSection(models.Model):
     document = models.ForeignKey(
         Document,
         verbose_name=_('Parent document'),
-        related_name='sections',
+        related_name='document_sections',
         )
 
     position = models.IntegerField(
@@ -485,13 +449,12 @@ class DocumentSection(models.Model):
 
     def move_to(self, pos):
         try:
-            existing_position = DocumentSection.objects.get(document=self.document, position=pos)
+            DocumentSection.objects.get(document=self.document, position=pos)
         except DocumentSection.DoesNotExist:
             raise ValueError(_('Can not move to non existing position'))
 
-
         if pos > self.position:
-            for i in DocumentSection.objects.filter(document=self.document, position__in=range(self.position+1, pos+1)):
+            for i in DocumentSection.objects.filter(document=self.document, position__in=range(self.position + 1, pos + 1)):
                 i.position -= 1
                 i._flexisave()
         elif pos < self.position:
@@ -508,7 +471,7 @@ class DocumentSection(models.Model):
         if self.position == None:
             self.position = count
         elif self.position >= count:
-            self.position = count-1
+            self.position = count - 1
         try:
             existing_position = DocumentSection.objects.get(document=self.document, position=self.position)
             if existing_position.id != self.id:
@@ -535,11 +498,24 @@ class CustomStyle(models.Model):
 
     css_chunk = models.TextField(
         verbose_name=_('css chunk'),
+        null=True,
+        blank=True,
+    )
+
+
+class CustomStyleImage(models.Model):
+    customstyle = models.ForeignKey(CustomStyle)
+
+    custom_css_image = models.ImageField(
+        verbose_name=('Custom CSS image'),
+        upload_to='section_images',
+        null=True,
+        blank=True,
     )
 
 
 def create_menus(sender, **kwargs):
-    if issubclass(sender, Section):
+    if issubclass(sender, BaseSection):
         created = kwargs.get('created', False)
         instance = kwargs.get('instance')
 
@@ -559,11 +535,11 @@ def handle_link_url_post_save(sender, instance, **kwargs):
     linklist = []
     if isinstance(instance, BaseLink):
         linklist = [instance]
-    elif isinstance(instance, BaseContent):
-        linklist = instance.contentlink_set.all()
     elif isinstance(instance, BaseSection):
         for content in instance.related_content.all():
             linklist += content.contentlink_set.all()
+    elif isinstance(instance, BaseContent):
+        linklist = instance.contentlink_set.all()
 
     for link in linklist:
         if link and link.get_absolute_url() != link.menu.url:
