@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -23,20 +23,32 @@ from django.template import RequestContext
 from django.utils import simplejson
 from django.utils.translation import get_language_from_request
 
-from searchform.registry import search_form_registry
-from merengue.section.models import BaseSection, Document, Section, \
-                                    DocumentSection
+from merengue.section.models import BaseSection, Document, \
+                                    DocumentSection, BaseLink
 
 from merengue.base.views import content_view
 from merengue.section.models import AbsoluteLink, ContentLink, ViewletLink, Menu
 
+from merengue.perms import utils as perms_api
+from merengue.base.utils import get_render_http_error
+
+
+def section_index(request):
+    """ place holder to have reverse URL resolution """
+    return HttpResponse('')
+
 
 def section_view(request, section_slug, original_context={},
                  template='section/section_view_without_maincontent.html'):
     section_slug = section_slug.strip('/')
     section = get_object_or_404(BaseSection, slug=section_slug)
+    has_view = perms_api.has_permission(section, request.user, 'view')
+    if not has_view:
+        forbidden_response = HttpResponse()
+        forbidden_response.write(get_render_http_error(request, 403))
+        return forbidden_response
     context = original_context or {}
-    context['section'] = section.real_instance
+    context['section'] = section.get_real_instance()
     main_content = section.main_content and section.main_content.get_real_instance() or None
     if not main_content:
         return section_view_without_maincontent(request, context, template)
@@ -77,13 +89,17 @@ def menu_section_view(request, section_slug, menu_slug):
         except Menu.DoesNotExist:
             raise Http404
 
-    link = menu.baselink.real_instance
+    try:
+        link = menu.baselink.real_instance
+    except BaseLink.DoesNotExist:
+        return render_to_response('section/menu_link_not_exists.html',
+            {'menu': menu}, context_instance=RequestContext(request))
     if isinstance(link, AbsoluteLink):
-        return HttpResponseRedirect(link.url)
+        return HttpResponseRedirect(link.get_absolute_url())
     else:
         context = {}
         if section_slug:
-            context['section'] = section.real_instance
+            context['section'] = section.get_real_instance()
         context['menu'] = menu
         if isinstance(link, ViewletLink):
             context['registered_viewlet'] = link.viewlet
@@ -107,7 +123,7 @@ def section_view_without_maincontent(request, context,
     admin_absolute_url = False
     if user.is_authenticated():
         if user.is_superuser or (user.is_staff and
-                                  (user.has_perm('section.change_%s'% section._meta.module_name) or
+                                 (user.has_perm('section.change_%s' % section._meta.module_name) or
                                   user.has_perm('section.change_basesection'))):
             admin_absolute_url = True
     context['admin_absolute_url'] = admin_absolute_url
@@ -116,61 +132,13 @@ def section_view_without_maincontent(request, context,
 
 
 def section_custom_style(request, section_slug):
-    section = get_object_or_404(Section, slug=section_slug)
+    section = get_object_or_404(BaseSection, slug=section_slug)
     return render_to_response('section/section_colors.css',
                               {'customstyle': section.customstyle},
                               context_instance=RequestContext(request),
                               mimetype='text/css')
 
 
-def _parse_search_form_filters(value):
-    filters={}
-    for filter_and_options in value.strip().split('\n'):
-        if filter_and_options:
-            (filter_name, options) = filter_and_options.split(':')
-            filter_name = filter_name.strip()
-            option_list=[]
-            for option in options.strip().split(','):
-                soption=option.strip()
-                if soption.isdigit():
-                    option_list.append(int(soption))
-                else:
-                    option_list.append(soption)
-            filters.update({filter_name: option_list})
-    return filters
-
-
-@login_required
-def get_search_filters_and_options(request):
-    search_form = request.GET.get('search_form', None)
-    widget_name = request.GET.get('widget_name', 'search_form_filters')
-    value = request.GET.get('value', '')
-    results = []
-    actual_filters = _parse_search_form_filters(value)
-    if search_form:
-        search_form_class = search_form_registry.get_form_class(search_form)
-        for filter in search_form_registry.get_filters(search_form):
-            options=[]
-            selected_options = actual_filters.get(filter, [])
-            for (option_value, option_name) in search_form_registry.get_filter_options(search_form, filter):
-                selected = option_value in selected_options
-                options.append({'name': option_name,
-                                'value': option_value,
-                                'selected': selected,
-                               })
-            results.append({
-                'label': search_form_class.fields[filter].label,
-                'name': filter,
-                'options': options,
-            })
-    return render_to_response('section/search_form_filters.html',
-                              {'filters': results,
-                               'widget_name': widget_name,
-                               'value': value,
-                              },
-                              context_instance=RequestContext(request))
-
-
 @login_required
 def save_menu_order(request):
     for key in request.GET.keys():
@@ -206,7 +174,7 @@ def section_dispatcher(request, url):
         func = menu_section_view
         kwargs.update({'menu_slug': parts[-1]})
     elif not url.startswith('/sections'):
-        return HttpResponseRedirect('/sections%s' %url)
+        return HttpResponseRedirect('/sections%s' % url)
     else:
         raise Http404
     return func(request, **kwargs)
@@ -234,7 +202,6 @@ def insert_document_section_after(request):
 
 @login_required
 def document_section_delete(request):
-    document_id = request.GET.get('document_id', None)
     document_section_id = request.GET.get('document_section_id', None)
     section = get_object_or_404(DocumentSection, id=document_section_id)
     section.delete()
@@ -244,7 +211,6 @@ def document_section_delete(request):
 
 @login_required
 def document_section_edit(request):
-    document_id = request.POST.get('document_id', None)
     document_section_id = request.POST.get('document_section_id', None)
     document_section_body = request.POST.get('document_section_body', None)
     section = get_object_or_404(DocumentSection, id=document_section_id)
@@ -257,8 +223,6 @@ def document_section_edit(request):
 
 @login_required
 def document_section_move(request):
-    document_id = request.GET.get('document_id', None)
-    document = get_object_or_404(Document, id=document_id)
     document_section_id = request.GET.get('document_section_id', None)
     prevsection = request.GET.get('document_section_prev', None)
     nextsection = request.GET.get('document_section_next', None)
