@@ -59,6 +59,8 @@ MERENGUE_APPS = (
     'merengue.portal',
     'merengue.internallinks',
     'merengue.collection',
+    'merengue.review',
+    'merengue.uitools',
     'merengue.collab',  # Please, keep this application in last place cause it's used to know when activate plugins after migration
 )
 
@@ -100,6 +102,8 @@ INSTALLED_APPS = (
     'oembed',
     'ajax_select',
     'notification',
+    'compressor',
+    'announcements',
 ) + MERENGUE_APPS
 
 # merengue exclusive middlewares. you have to put at least these middleware in your project settings
@@ -107,6 +111,7 @@ MERENGUE_MIDDLEWARE_CLASSES = (
     'merengue.middleware.RemoveRandomAjaxParameter',
     'merengue.section.middleware.RequestSectionMiddleware',
     'merengue.section.middleware.DebugSectionMiddleware',
+    'merengue.multimedia.middleware.MediaMiddleware',
     'merengue.middleware.SimplifiedLayoutMiddleware',
     'merengue.middleware.LocaleMiddleware',
     'merengue.middleware.HttpStatusCodeRendererMiddleware',
@@ -114,8 +119,10 @@ MERENGUE_MIDDLEWARE_CLASSES = (
 )
 
 # merengue usual middleware list. you can use this variable in your MIDDLEWARE_CLASSES project settings
-MIDDLEWARE_CLASSES = (
+
+PRE_MERENGUE_MIDDLEWARE_CLASSES = (
     'johnny.middleware.LocalStoreClearMiddleware',  # this has to be first
+    #'cmsutils.middleware.ProfileMiddleware', # remove comment if you want to profile your website
     #'cmsutils.middleware.I18NUpdateCacheMiddleware', # removed anonymous cache middleware
     'johnny.middleware.QueryCacheMiddleware',
     'django.middleware.common.CommonMiddleware',
@@ -125,20 +132,26 @@ MIDDLEWARE_CLASSES = (
     'pagination.middleware.PaginationMiddleware',
     'django.middleware.locale.LocaleMiddleware',
     'cmsutils.middleware.AutomatizedTestingMiddleware',
-) + MERENGUE_MIDDLEWARE_CLASSES + (
+)
+
+POST_MERENGUE_MIDDLEWARE_CLASSES = (
     #'django.middleware.gzip.GZipMiddleware',
     'debug_toolbar.middleware.DebugToolbarMiddleware',
     'merengue.section.middleware.ResponseSectionMiddleware',
     'merengue.pluggable.middleware.PluginMiddlewaresProxy',
+    'merengue.block.middleware.RenderBlockMiddleware',
     #'cmsutils.middleware.I18NFetchFromCacheMiddleware', # this has to be last # removed anonymous cache middleware
 )
 
+MIDDLEWARE_CLASSES = PRE_MERENGUE_MIDDLEWARE_CLASSES + MERENGUE_MIDDLEWARE_CLASSES + POST_MERENGUE_MIDDLEWARE_CLASSES
+
 # merengue status list for contents workflow.
 STATUS_LIST = (
     ('draft', ugettext('Borrador')),
     ('pending', ugettext('Pendiente')),
     ('published', ugettext('Publicado')),
 )
+DEFAULT_STATUS = 'draft'
 
 # merengue default directory for plugins
 PLUGINS_DIR = 'plugins'
@@ -155,6 +168,9 @@ DETECT_BROKEN_PLUGINS = True
 # The module to store session data
 SESSION_ENGINE = 'merengue.backends.db'
 
+# Users can manage sections they own and their related objects
+ACQUIRE_SECTION_OWNERSHIP = False
+
 # cache default settings
 CACHE_BACKEND = 'johnny.backends.locmem:///'
 CACHE_MIDDLEWARE_SECONDS = 3600 * 24
@@ -192,29 +208,21 @@ CAPTCHA_SETTINGS = {
                     'B': '8',
     },
     'CASE_SENSITIVE': False,
+    'USE_EXTRA_IMAGE': False,
 }
 
 # Default parameters for suggestion box
 SUGGESTION_BOX_EMAIL = 'info@foo.com'
 CONTACT_SUGGESTIONBOX_PREFIX = 'SUGGESTION BOX'
 
+# For transhette
+ENABLE_TRANSLATION_SUGGESTIONS = False
 
 # Map parameters
 MAP_FILTRABLE_MODELS = (
     'base.basecontent',
 )
 
-DEBUG_TOOLBAR_PANELS = (
-    #'debug_toolbar.panels.version.VersionDebugPanel',
-    #'debug_toolbar.panels.timer.TimerDebugPanel',
-    #'debug_toolbar.panels.headers.HeaderDebugPanel',
-    #'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
-    #'debug_toolbar.panels.sql.SQLDebugPanel',
-    #'debug_toolbar.panels.cache.CacheDebugPanel',
-    'debug_toolbar.panels.template.TemplateDebugPanel',
-    #'debug_toolbar.panels.logger.LoggingPanel',
-)
-
 DEBUG_TOOLBAR_EXCLUDED_URLS = (
     r'^/tinyimages/',
 )
@@ -262,7 +270,7 @@ MANAGE_FILE = 'manage.py'
 # /usr/bin/python
 SYS_EXECUTABLE = None
 
-JOHNNY_TABLE_BLACKLIST = ('south_migrationhistory', )
+JOHNNY_TABLE_BLACKLIST = ('south_migrationhistory', 'django_content_type', )
 
 # Path for translation catalogs search
 LOCALE_PATHS = (
@@ -273,3 +281,12 @@ LOCALE_PATHS = (
 DEFAULT_THEME_PREVIEW = 'merengue/img/preview_merengue.png'
 
 STATIC_ROLES = [u'Anonymous User', u'Owner']
+
+# inplaceeditform
+ADAPTOR_INPLACEEDIT = {'textarea': 'merengue.uitools.fields.AdaptorTinyMCEField'}
+
+# Plugin default preview image. The real path would be path.join(MEDIA_URL, DEFAULT_PLUGIN_PREVIEW)
+DEFAULT_PLUGIN_PREVIEW = path.join('merengue', 'img', 'preview_merengue.png')
+
+# Num element per page in collections
+NUM_ELEM_PER_PAGE_DEFAULT = 10
